// use std::io::BufReader;
// use std::path::PathBuf;
// use std::thread;
// use std::time::Duration;
extern crate rodio;
mod base;
use base::*;
use imgui::*;
use walkdir::WalkDir;
mod support_ogl;
// mod support_glium;
use env_logger;
use log::*;

fn collect_sounds (root: &str) -> Vec<Sound> {
    WalkDir::new(root)
    .into_iter()
    .filter_map(|e| e.ok())
    .filter(|e| e.path().is_file())
    .map(|f| Sound::new(&f.path().to_string_lossy().to_string()))
    .filter_map(|e| e.ok())
    .collect()
}
fn main() {
    env_logger::init();
    let dev = rodio::default_output_device().unwrap();
    
    let mut sounds = collect_sounds("media");
    
    let mut timeline = Timeline::default();
    // let mut track = Track::default();
    // track.test();
    // track.bar_mut(0).bpm = 1;
    // track.bars[0].bpm = 1;
    // let mut x = &mut track.bars[0];
    // x.bpm = 1;

    // dbg!(&track);

    // return;
    let mut active_sound: usize = 0;
    let mut active_track: usize = 0;
    // let mut active_bar: usize = 0;


    // let mut active_bar = Bar::default();
    let mut active_bar = &mut timeline.tracks[0].bars[0];


    let mut system = support_ogl::init(file!());
    let s = system.imgui.style_mut();
    s.window_rounding = 1.5;
    // let system = support_glium::init(file!());

    system.main_loop(move |_, ui| {
        Window::new(im_str!("sources"))
            // .resizable(false)
            .position([400.0, 140.0], Condition::Appearing)
            .collapsible(false)
            // .always_auto_resize(true)
            .size([300.0, 300.0], Condition::FirstUseEver)
            .build(ui, || {

                // ui.show_default_style_editor();

                ui.tree_node(im_str!("snds"))
                .opened(true, Condition::Appearing)
                .build(|| {
                    for s in &mut sounds {
                        ui.tree_node(&im_str!("{}", s.name)).build(|| {
                            // ui.same_line(0.0);
                            if ui.small_button(im_str!("add to bar")) {
                                active_bar.sounds[active_sound] = Some(s.clone());
                            }
                            ui.same_line(0.0);
                            if ui.small_button(im_str!("play")) {
                                s.play(&dev);
                            }
                        });
                    }
                });
            });

        Window::new(im_str!("bar"))
            .size([300.0, 100.0], Condition::FirstUseEver)
            .build(ui, || {

                for i in 0..active_bar.length() {
                    ui.same_line(0.0);
                    
                    let token = match active_sound == i {
                        true => ui.push_style_color(StyleColor::Button, [1.0, 0.0, 0.0, 1.0]),
                        false => ui.push_style_color(StyleColor::CheckMark, [0.0, 0.0, 0.0, 1.0])
                    };
                    if ui.small_button(&im_str!("{}", i)) {
                        active_sound = i as usize;
                    }
                    token.pop(ui);

                }
                if ui.small_button(&im_str!("extend")) {
                    active_bar.extend(1);
                }

                
                if ui.small_button(&im_str!("ply")) {
                    active_bar.play(&dev);
                }
            });
    
            Window::new(im_str!("sound"))
            .size([300.0, 100.0], Condition::FirstUseEver)
            .build(ui, || {
                let snd = &active_bar.sounds[active_sound];
                if let Some(s) = snd{
                    ui.text(im_str!("name   {}", s.name));
                    ui.text(im_str!("roll   {}", s.roll));
                    ui.text(im_str!("volume {}", s.volume));
                    ui.text(im_str!("rate   {}", s.rate));
                }
            });
    
            Window::new(im_str!("tracks"))
            .size([300.0, 100.0], Condition::FirstUseEver)
            .build(ui, || {
               
                if ui.small_button(&im_str!("add")) {
                    active_bar.play(&dev);
                }
            });
    
    
        });
}
